@@ -1,5 +1,6 @@
 /* BFD back-end data structures for a.out (and similar) files.
-   Copyright 1990, 1991, 1992 Free Software Foundation, Inc.
+   Copyright 1990, 91, 92, 93, 94, 95, 96, 97, 1998
+   Free Software Foundation, Inc.
    Written by Cygnus Support.
 
 This file is part of BFD, the Binary File Descriptor library.
@@ -16,12 +17,17 @@ GNU General Public License for more details.
 
 You should have received a copy of the GNU General Public License
 along with this program; if not, write to the Free Software
-Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.  */
+Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.  */
+
+#ifndef LIBAOUT_H
+#define LIBAOUT_H
 
 /* We try to encapsulate the differences in the various a.out file
    variants in a few routines, and otherwise share large masses of code.
    This means we only have to fix bugs in one place, most of the time.  */
 
+#include "bfdlink.h"
+
 /* Parameterize the a.out code based on whether it is being built
    for a 32-bit architecture or a 64-bit architecture.  */
 #if ARCH_SIZE==64
@@ -50,7 +56,44 @@ struct external_exec;
 struct external_nlist;
 struct reloc_ext_external;
 struct reloc_std_external;
+
+/* a.out backend linker hash table entries.  */
 
+struct aout_link_hash_entry
+{
+  struct bfd_link_hash_entry root;
+  /* Whether this symbol has been written out.  */
+  boolean written;
+  /* Symbol index in output file.  */
+  int indx;
+};
+
+/* a.out backend linker hash table.  */
+
+struct aout_link_hash_table
+{
+  struct bfd_link_hash_table root;
+};
+
+/* Look up an entry in an a.out link hash table.  */
+
+#define aout_link_hash_lookup(table, string, create, copy, follow) \
+  ((struct aout_link_hash_entry *) \
+   bfd_link_hash_lookup (&(table)->root, (string), (create), (copy), (follow)))
+
+/* Traverse an a.out link hash table.  */
+
+#define aout_link_hash_traverse(table, func, info)			\
+  (bfd_link_hash_traverse						\
+   (&(table)->root,							\
+    (boolean (*) PARAMS ((struct bfd_link_hash_entry *, PTR))) (func),	\
+    (info)))
+
+/* Get the a.out link hash table from the info structure.  This is
+   just a cast.  */
+
+#define aout_hash_table(p) ((struct aout_link_hash_table *) ((p)->hash))
+
 /* Back-end information for various a.out targets.  */
 struct aout_backend_data
 {
@@ -63,6 +106,12 @@ struct aout_backend_data
      If not, the text section starts on the next page.  */
   unsigned char text_includes_header;
 
+  /* If this flag is set, then if the entry address is not in the
+     first SEGMENT_SIZE bytes of the text section, it is taken to be
+     the address of the start of the text section.  This can be useful
+     for kernels.  */
+  unsigned char entry_is_text_address;
+
   /* The value to pass to N_SET_FLAGS.  */
   unsigned char exec_hdr_flags;
 
@@ -82,6 +131,44 @@ struct aout_backend_data
      to the size of the text section in the file for alignment purposes but
      does *not* get counted in the length of the text section. */
   unsigned char exec_header_not_counted;
+
+  /* Callback from the add symbols phase of the linker code to handle
+     a dynamic object.  */
+  boolean (*add_dynamic_symbols) PARAMS ((bfd *, struct bfd_link_info *,
+					  struct external_nlist **,
+					  bfd_size_type *, char **));
+
+  /* Callback from the add symbols phase of the linker code to handle
+     adding a single symbol to the global linker hash table.  */
+  boolean (*add_one_symbol) PARAMS ((struct bfd_link_info *, bfd *,
+				     const char *, flagword, asection *,
+				     bfd_vma, const char *, boolean,
+				     boolean,
+				     struct bfd_link_hash_entry **));
+
+  /* Called to handle linking a dynamic object.  */
+  boolean (*link_dynamic_object) PARAMS ((struct bfd_link_info *, bfd *));
+
+  /* Called for each global symbol being written out by the linker.
+     This should write out the dynamic symbol information.  */
+  boolean (*write_dynamic_symbol) PARAMS ((bfd *, struct bfd_link_info *,
+					   struct aout_link_hash_entry *));
+
+  /* If this callback is not NULL, the linker calls it for each reloc.
+     RELOC is a pointer to the unswapped reloc.  If *SKIP is set to
+     true, the reloc will be skipped.  *RELOCATION may be changed to
+     change the effects of the relocation.  */
+  boolean (*check_dynamic_reloc) PARAMS ((struct bfd_link_info *info,
+					  bfd *input_bfd,
+					  asection *input_section,
+					  struct aout_link_hash_entry *h,
+					  PTR reloc, bfd_byte *contents,
+					  boolean *skip,
+					  bfd_vma *relocation));
+
+  /* Called at the end of a link to finish up any dynamic linking
+     information.  */
+  boolean (*finish_dynamic_link) PARAMS ((bfd *, struct bfd_link_info *));
 };
 #define aout_backend_info(abfd) \
 	((CONST struct aout_backend_data *)((abfd)->xvec->backend_data))
@@ -112,14 +199,14 @@ struct internal_exec
 };
 
 /* Magic number is written 
-< MSB        >
+< MSB          >
 3130292827262524232221201918171615141312111009080706050403020100
-< FLAGS      >< MACHINE TYPE ><  MAGIC NUMBER		       >
+< FLAGS        >< MACHINE TYPE ><  MAGIC NUMBER                >
 */
 /* Magic number for NetBSD is
-<MSB         >
+<MSB           >
 3130292827262524232221201918171615141312111009080706050403020100
-< FLAGS    ><                  ><  MAGIC NUMBER                >
+< FLAGS    >< MACHINE TYPE     ><  MAGIC NUMBER                >
 */
 
 enum machine_type {
@@ -128,15 +215,35 @@ enum machine_type {
   M_68020 = 2,
   M_SPARC = 3,
   /* skip a bunch so we don't run into any of suns numbers */
+  /* make these up for the ns32k*/
+  M_NS32032 = (64),		/* ns32032 running ? */
+  M_NS32532 = (64 + 5),		/* ns32532 running mach */
+
   M_386 = 100,
   M_29K = 101,          /* AMD 29000 */
   M_386_DYNIX = 102,	/* Sequent running dynix */
-  M_386_NETBSD = 134,		/* NetBSD/386 binary */
+  M_ARM = 103,		/* Advanced Risc Machines ARM */
+  M_SPARCLET = 131,	/* SPARClet = M_SPARC + 128 */
+  M_386_NETBSD = 134,	/* NetBSD/i386 binary */
+  M_68K_NETBSD = 135,	/* NetBSD/m68k binary */
+  M_68K4K_NETBSD = 136,	/* NetBSD/m68k4k binary */
+  M_532_NETBSD = 137,	/* NetBSD/ns32k binary */
+  M_SPARC_NETBSD = 138,	/* NetBSD/sparc binary */
+  M_PMAX_NETBSD = 139,	/* NetBSD/pmax (MIPS little-endian) binary */
+  M_VAX_NETBSD = 140,	/* NetBSD/vax binary */
+  M_ALPHA_NETBSD = 141,	/* NetBSD/alpha binary */
+  M_SPARCLET_1 = 147,	/* 0x93, reserved */
   M_MIPS1 = 151,        /* MIPS R2000/R3000 binary */
   M_MIPS2 = 152,        /* MIPS R4000/R6000 binary */
+  M_SPARCLET_2 = 163,	/* 0xa3, reserved */
+  M_SPARCLET_3 = 179,	/* 0xb3, reserved */
+  M_SPARCLET_4 = 195,	/* 0xc3, reserved */
   M_HP200 = 200,	/* HP 200 (68010) BSD binary */
   M_HP300 = (300 % 256), /* HP 300 (68020+68881) BSD binary */
-  M_HPUX = (0x20c % 256)/* HP 200/300 HPUX binary */
+  M_HPUX = (0x20c % 256), /* HP 200/300 HPUX binary */
+  M_SPARCLET_5 = 211,	/* 0xd3, reserved */
+  M_SPARCLET_6 = 227,	/* 0xe3, reserved */
+  M_SPARCLET_7 = 243	/* 0xf3, reserved */
 };
 
 #define N_DYNAMIC(exec) ((exec).a_info & 0x80000000)
@@ -220,29 +327,50 @@ struct aoutdata {
   /* Segment size - needed for alignment of demand paged files. */
   unsigned long segment_size;
 
+  /* Zmagic disk block size - need to align the start of the text
+     section in ZMAGIC binaries.  Normally the same as page_size.  */
+  unsigned long zmagic_disk_block_size;
+
   unsigned exec_bytes_size;
   unsigned vma_adjusted : 1;
 
   /* used when a bfd supports several highly similar formats */
-  enum {
-    default_format = 0,
-    gnu_encap_format } subformat;
+  enum
+    {
+      default_format = 0,
+      /* Used on HP 9000/300 running HP/UX.  See hp300hpux.c.  */
+      gnu_encap_format,
+      /* Used on Linux, 386BSD, etc.  See include/aout/aout64.h.  */
+      q_magic_format
+    } subformat;
 
-  enum {
-    undecided_magic = 0,
-    z_magic,
-    o_magic,
-    n_magic } magic;
+  enum
+    {
+      undecided_magic = 0,
+      z_magic,
+      o_magic,
+      n_magic
+    } magic;
+
+  /* A buffer for find_nearest_line.  */
+  char *line_buf;
 
   /* The external symbol information.  */
   struct external_nlist *external_syms;
   bfd_size_type external_sym_count;
+  bfd_window sym_window;
   char *external_strings;
   bfd_size_type external_string_size;
+  bfd_window string_window;
   struct aout_link_hash_entry **sym_hashes;
 
   /* A pointer for shared library information.  */
   PTR dynamic_info;
+
+  /* A mapping from local symbols to offsets into the global offset
+     table, used when linking on SunOS.  This is indexed by the symbol
+     index.  */
+  bfd_vma *local_got_offsets;
 };
 
 struct  aout_data_struct {
@@ -263,8 +391,10 @@ struct  aout_data_struct {
 #define obj_aout_subformat(bfd)	(adata(bfd).subformat)
 #define obj_aout_external_syms(bfd) (adata(bfd).external_syms)
 #define obj_aout_external_sym_count(bfd) (adata(bfd).external_sym_count)
+#define obj_aout_sym_window(bfd) (adata(bfd).sym_window)
 #define obj_aout_external_strings(bfd) (adata(bfd).external_strings)
 #define obj_aout_external_string_size(bfd) (adata(bfd).external_string_size)
+#define obj_aout_string_window(bfd) (adata(bfd).string_window)
 #define obj_aout_sym_hashes(bfd) (adata(bfd).sym_hashes)
 #define obj_aout_dynamic_info(bfd) (adata(bfd).dynamic_info)
 
@@ -272,6 +402,21 @@ struct  aout_data_struct {
    macro is only ever applied to an asymbol */
 #define aout_symbol(asymbol) ((aout_symbol_type *)(&(asymbol)->the_bfd))
 
+/* Information we keep for each a.out section.  This is currently only
+   used by the a.out backend linker.  */
+
+struct aout_section_data_struct
+{
+  /* The unswapped relocation entries for this section.  */
+  PTR relocs;
+};
+
+#define aout_section_data(s) \
+  ((struct aout_section_data_struct *) (s)->used_by_bfd)
+
+#define set_aout_section_data(s,v) \
+  ((s)->used_by_bfd = (PTR)&(v)->relocs)
+
 /* Prototype declarations for functions defined in aoutx.h  */
 
 boolean
@@ -280,17 +425,18 @@ NAME(aout,squirt_out_relocs) PARAMS ((bfd *abfd, asection *section));
 boolean
 NAME(aout,make_sections) PARAMS ((bfd *));
 
-bfd_target *
+const bfd_target *
 NAME(aout,some_aout_object_p) PARAMS ((bfd *abfd,
-			struct internal_exec *execp,
-			bfd_target * (*callback)(bfd *)));
+				       struct internal_exec *execp,
+				       const bfd_target *(*callback)(bfd *)));
 
 boolean
 NAME(aout,mkobject) PARAMS ((bfd *abfd));
 
 enum machine_type
 NAME(aout,machine_type) PARAMS ((enum bfd_architecture arch,
-				 unsigned long machine));
+				 unsigned long machine,
+				 boolean *unknown));
 
 boolean
 NAME(aout,set_arch_mach) PARAMS ((bfd *abfd, enum bfd_architecture arch,
@@ -330,10 +476,14 @@ NAME(aout,get_symtab) PARAMS ((bfd *abfd, asymbol **location));
 
 void
 NAME(aout,swap_ext_reloc_in) PARAMS ((bfd *, struct reloc_ext_external *,
-				      arelent *, asymbol **));
+				      arelent *, asymbol **, bfd_size_type));
 void
 NAME(aout,swap_std_reloc_in) PARAMS ((bfd *, struct reloc_std_external *,
-				      arelent *, asymbol **));
+				      arelent *, asymbol **, bfd_size_type));
+
+reloc_howto_type *
+NAME(aout,reloc_type_lookup) PARAMS ((bfd *abfd,
+				      bfd_reloc_code_real_type code));
 
 boolean
 NAME(aout,slurp_reloc_table) PARAMS ((bfd *abfd, sec_ptr asect,
@@ -365,6 +515,13 @@ NAME(aout,find_nearest_line) PARAMS ((bfd *abfd, asection *section,
       asymbol **symbols, bfd_vma offset, CONST char **filename_ptr,
       CONST char **functionname_ptr, unsigned int *line_ptr));
 
+long
+NAME(aout,read_minisymbols) PARAMS ((bfd *, boolean, PTR *, unsigned int *));
+
+asymbol *
+NAME(aout,minisymbol_to_symbol) PARAMS ((bfd *, boolean, const PTR,
+					 asymbol *));
+
 int
 NAME(aout,sizeof_headers) PARAMS ((bfd *abfd, boolean exec));
 
@@ -380,6 +537,17 @@ void
 NAME(aout,swap_exec_header_out) PARAMS ((bfd *abfd,
        struct internal_exec *execp, struct external_exec *raw_bytes));
 
+struct bfd_hash_entry *
+NAME(aout,link_hash_newfunc)
+  PARAMS ((struct bfd_hash_entry *, struct bfd_hash_table *, const char *));
+
+boolean
+NAME(aout,link_hash_table_init)
+     PARAMS ((struct aout_link_hash_table *, bfd *,
+	      struct bfd_hash_entry *(*) (struct bfd_hash_entry *,
+					  struct bfd_hash_table *,
+					  const char *)));
+
 struct bfd_link_hash_table *
 NAME(aout,link_hash_table_create) PARAMS ((bfd *));
 
@@ -394,11 +562,6 @@ NAME(aout,final_link) PARAMS ((bfd *, struct bfd_link_info *,
 boolean
 NAME(aout,bfd_free_cached_info) PARAMS ((bfd *));
 
-/* Prototypes for functions in stab-syms.c. */
-
-CONST char *
-aout_stab_name PARAMS ((int code));
-
 /* A.out uses the generic versions of these routines... */
 
 #define	aout_32_get_section_contents	_bfd_generic_get_section_contents
@@ -408,8 +571,8 @@ aout_stab_name PARAMS ((int code));
 #define NO_WRITE_HEADER_KLUDGE 0
 #endif
 
-#ifndef aout_32_bfd_is_local_label
-#define aout_32_bfd_is_local_label bfd_generic_is_local_label
+#ifndef aout_32_bfd_is_local_label_name
+#define aout_32_bfd_is_local_label_name bfd_generic_is_local_label_name
 #endif
 
 #ifndef WRITE_HEADERS
@@ -437,25 +600,23 @@ aout_stab_name PARAMS ((int code));
   									      \
 	if (bfd_get_outsymbols (abfd) != (asymbol **) NULL		      \
 	    && bfd_get_symcount (abfd) != 0) 				      \
-	    {								      \
-	      if (bfd_seek (abfd, (file_ptr)(N_SYMOFF(*execp)), SEEK_SET)     \
-		  != 0)							      \
-	        return false;						      \
+	  {								      \
+	    if (bfd_seek (abfd, (file_ptr)(N_SYMOFF(*execp)), SEEK_SET) != 0) \
+	      return false;						      \
 									      \
-	      if (! NAME(aout,write_syms)(abfd)) return false;		      \
+	    if (! NAME(aout,write_syms)(abfd)) return false;		      \
+	  }								      \
 									      \
-	      if (bfd_seek (abfd, (file_ptr)(N_TRELOFF(*execp)), SEEK_SET)    \
-		  != 0)							      \
-	        return false;						      \
+	if (bfd_seek (abfd, (file_ptr)(N_TRELOFF(*execp)), SEEK_SET) != 0)    \
+	  return false;						      	      \
+	if (!NAME(aout,squirt_out_relocs) (abfd, obj_textsec (abfd)))         \
+	  return false;						      	      \
 									      \
-	      if (!NAME(aout,squirt_out_relocs) (abfd, obj_textsec (abfd)))   \
-		return false;						      \
-	      if (bfd_seek (abfd, (file_ptr)(N_DRELOFF(*execp)), SEEK_SET)    \
-		  != 0)							      \
-	        return false;						      \
-									      \
-	      if (!NAME(aout,squirt_out_relocs)(abfd, obj_datasec (abfd)))    \
-		return false;						      \
-	    }								      \
+	if (bfd_seek (abfd, (file_ptr)(N_DRELOFF(*execp)), SEEK_SET) != 0)    \
+	  return false;						      	      \
+	if (!NAME(aout,squirt_out_relocs)(abfd, obj_datasec (abfd)))          \
+	  return false;						      	      \
       }									      
 #endif
+
+#endif /* ! defined (LIBAOUT_H) */
