@@ -1,6 +1,5 @@
 /* coffgrok.c
-
-   Copyright (C) 1994 Free Software Foundation, Inc.
+   Copyright (C) 1994, 95, 97, 1998 Free Software Foundation, Inc.
 
 This file is part of GNU Binutils.
 
@@ -16,7 +15,7 @@ GNU General Public License for more details.
 
 You should have received a copy of the GNU General Public License
 along with this program; if not, write to the Free Software
-Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.  */
+Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.  */
 
 /* Written by Steve Chamberlain (sac@cygnus.com)
 
@@ -27,12 +26,12 @@ Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.  */
 */
 
 #include <bfd.h>
-#include <stdio.h>
+#include "bucomm.h"
 
 #include "coff/internal.h"
 #include "../bfd/libcoff.h"
 #include "coffgrok.h"
-int lofile = -1;
+int lofile = 1;
 static struct coff_scope *top_scope;
 static struct coff_scope *file_scope;
 static struct coff_ofile *ofile;
@@ -54,8 +53,7 @@ static long symcount;
 static struct coff_ptr_struct *rawsyms;
 static int rawcount;
 static bfd *abfd;
-extern char *xcalloc();
-extern char *xmalloc();
+extern char *xcalloc ();
 #define PTR_SIZE 	4
 #define SHORT_SIZE 	2
 #define INT_SIZE 	4
@@ -82,23 +80,25 @@ empty_symbol ()
 /*int l;*/
 static void
 push_scope (link)
+     int link;
 {
   struct coff_scope *n = empty_scope ();
-  if (link) {
-    if (top_scope)
-      {
-	if (top_scope->list_tail)
-	  {
-	    top_scope->list_tail->next = n;
-	  }
-	else
-	  {
-	    top_scope->list_head = n;
-	  }
-	top_scope->list_tail = n;
-      }
-  }
-    n->parent = top_scope;
+  if (link)
+    {
+      if (top_scope)
+	{
+	  if (top_scope->list_tail)
+	    {
+	      top_scope->list_tail->next = n;
+	    }
+	  else
+	    {
+	      top_scope->list_head = n;
+	    }
+	  top_scope->list_tail = n;
+	}
+    }
+  n->parent = top_scope;
 
   top_scope = n;
 }
@@ -116,7 +116,7 @@ do_sections_p1 (head)
   asection *section;
   int idx;
   struct coff_section *all = (struct coff_section *) (xcalloc (abfd->section_count + 1,
-							      sizeof (struct coff_section)));
+					     sizeof (struct coff_section)));
   head->nsections = abfd->section_count + 1;
   head->sections = all;
 
@@ -130,23 +130,25 @@ do_sections_p1 (head)
       relsize = bfd_get_reloc_upper_bound (abfd, section);
       if (relsize < 0)
 	bfd_fatal (bfd_get_filename (abfd));
+      if (relsize == 0)
+        continue;
       relpp = (arelent **) xmalloc (relsize);
       relcount = bfd_canonicalize_reloc (abfd, section, relpp, syms);
       if (relcount < 0)
 	bfd_fatal (bfd_get_filename (abfd));
 
-      head->sections[i].name = (char *)(section->name);
+      head->sections[i].name = (char *) (section->name);
       head->sections[i].code = section->flags & SEC_CODE;
       head->sections[i].data = section->flags & SEC_DATA;
       if (strcmp (section->name, ".bss") == 0)
 	head->sections[i].data = 1;
-      head->sections[i].address = section->vma;
+      head->sections[i].address = section->lma;
       head->sections[i].size = section->_raw_size;
       head->sections[i].number = idx;
       head->sections[i].nrelocs = section->reloc_count;
       head->sections[i].relocs =
 	(struct coff_reloc *) (xcalloc (section->reloc_count,
-				       sizeof (struct coff_reloc)));
+					sizeof (struct coff_reloc)));
       head->sections[i].bfd_section = section;
     }
   head->sections[0].name = "ABSOLUTE";
@@ -164,7 +166,8 @@ do_sections_p2 (head)
   asection *section;
   for (section = abfd->sections; section; section = section->next)
     {
-      int j;
+      unsigned int j;
+
       for (j = 0; j < section->reloc_count; j++)
 	{
 	  int idx;
@@ -184,16 +187,20 @@ do_where (i)
      int i;
 {
   struct internal_syment *sym = &rawsyms[i].u.syment;
-  struct coff_where *where = (struct coff_where *) (malloc (sizeof (struct coff_where)));
+  struct coff_where *where =
+    (struct coff_where *) (xmalloc (sizeof (struct coff_where)));
   where->offset = sym->n_value;
 
+  if (sym->n_scnum == -1)
+    sym->n_scnum = 0;
+
   switch (sym->n_sclass)
     {
-   case C_FIELD:
+    case C_FIELD:
       where->where = coff_where_member_of_struct;
       where->offset = sym->n_value / 8;
       where->bitoffset = sym->n_value % 8;
-      where->bitsize = rawsyms[i+1].u.auxent.x_sym.x_misc.x_lnsz.x_size;
+      where->bitsize = rawsyms[i + 1].u.auxent.x_sym.x_misc.x_lnsz.x_size;
       break;
     case C_MOE:
       where->where = coff_where_member_of_enum;
@@ -242,7 +249,8 @@ do_lines (i, name)
 {
   struct coff_line *res = (struct coff_line *) xcalloc (sizeof (struct coff_line), 1);
   asection *s;
-  int l;
+  unsigned int l;
+
   /* Find out if this function has any line numbers in the table */
   for (s = abfd->sections; s; s = s->next)
     {
@@ -289,9 +297,11 @@ do_type (i)
 {
   struct internal_syment *sym = &rawsyms[i].u.syment;
   union internal_auxent *aux = &rawsyms[i + 1].u.auxent;
-  struct coff_type *res = (struct coff_type *) malloc (sizeof (struct coff_type));
+  struct coff_type *res =
+    (struct coff_type *) xmalloc (sizeof (struct coff_type));
   int type = sym->n_type;
   int which_dt = 0;
+  int dimind = 0;
 
   res->type = coff_basic_type;
   res->u.basic = type & 0xf;
@@ -306,10 +316,23 @@ do_type (i)
 	  res->type = coff_secdef_type;
 	  res->size = aux->x_scn.x_scnlen;
 	}
-      else {
-	/* No clue what this is */
-      res->size = 0;
-    }
+      else
+	{
+	  if (type == 0)
+	    {
+	      /* Don't know what this is, let's make it a simple int */
+	      res->size = INT_SIZE;
+	      res->u.basic = T_UINT;
+	    }
+	  else
+	    {
+	      /* Else it could be a function or pointer to void */
+	      res->size = 0;
+	    }
+	}
+      break;
+
+
       break;
     case T_UCHAR:
     case T_CHAR:
@@ -351,6 +374,7 @@ do_type (i)
 	      last_struct = res;
 	      res->type = coff_structdef_type;
 	      res->u.astructdef.elements = empty_scope ();
+	      res->u.astructdef.idx = 0;
 	      res->u.astructdef.isstruct = (type & 0xf) == T_STRUCT;
 	      res->size = aux->x_sym.x_misc.x_lnsz.x_size;
 	    }
@@ -392,24 +416,23 @@ do_type (i)
 	  break;
 	case DT_ARY:
 	  {
-
-	    /* Look in the auxent for the dimensions and build lots of dims */
-	    int els = aux->x_sym.x_fcnary.x_ary.x_dimen[which_dt];
-	    if (els)
-	      {
-		struct coff_type *ptr = (struct coff_type *) malloc (sizeof (struct coff_type));
-		ptr->type = coff_array_type;
-		ptr->size = els * res->size;
-		ptr->u.array.dim = els;
-		ptr->u.array.array_of = res;
-		res = ptr;
-	      }
+	    struct coff_type *ptr = ((struct coff_type *)
+				     xmalloc (sizeof (struct coff_type)));
+	    int els = (dimind < DIMNUM
+		       ? aux->x_sym.x_fcnary.x_ary.x_dimen[dimind]
+		       : 0);
+	    ++dimind;
+	    ptr->type = coff_array_type;
+	    ptr->size = els * res->size;
+	    ptr->u.array.dim = els;
+	    ptr->u.array.array_of = res;
+	    res = ptr;
+	    break;
 	  }
-
-	  break;
 	case DT_PTR:
 	  {
-	    struct coff_type *ptr = (struct coff_type *) malloc (sizeof (struct coff_type));
+	    struct coff_type *ptr =
+	      (struct coff_type *) xmalloc (sizeof (struct coff_type));
 	    ptr->size = PTR_SIZE;
 	    ptr->type = coff_pointer_type;
 	    ptr->u.pointer.points_to = res;
@@ -418,7 +441,8 @@ do_type (i)
 	  }
 	case DT_FCN:
 	  {
-	    struct coff_type *ptr = (struct coff_type *) malloc (sizeof (struct coff_type));
+	    struct coff_type *ptr
+	      = (struct coff_type *) xmalloc (sizeof (struct coff_type));
 	    ptr->size = 0;
 	    ptr->type = coff_function_type;
 	    ptr->u.function.function_returns = res;
@@ -439,7 +463,8 @@ do_visible (i)
      int i;
 {
   struct internal_syment *sym = &rawsyms[i].u.syment;
-  struct coff_visible *visible = (struct coff_visible *) (malloc (sizeof (struct coff_visible)));
+  struct coff_visible *visible =
+    (struct coff_visible *) (xmalloc (sizeof (struct coff_visible)));
   enum coff_vis_type t;
   switch (sym->n_sclass)
     {
@@ -453,7 +478,7 @@ do_visible (i)
       break;
 
     case C_REGPARM:
-      t= coff_vis_regparam;
+      t = coff_vis_regparam;
       break;
 
     case C_REG:
@@ -467,7 +492,7 @@ do_visible (i)
       break;
     case C_AUTOARG:
     case C_ARG:
-      t  = coff_vis_autoparam;
+      t = coff_vis_autoparam;
       break;
     case C_AUTO:
 
@@ -511,16 +536,17 @@ do_define (i, b)
 
   s->number = ++symbol_index;
   s->name = sym->_n._n_nptr[1];
-
+  s->sfile = cur_sfile;
   /* Glue onto the ofile list */
-  if (lofile>=0) {
-    if (ofile->symbol_list_tail)
-      ofile->symbol_list_tail->next_in_ofile_list = s;
-    else
-      ofile->symbol_list_head = s;
-    ofile->symbol_list_tail = s;
-    /* And the block list */
-  }
+  if (lofile >= 0)
+    {
+      if (ofile->symbol_list_tail)
+	ofile->symbol_list_tail->next_in_ofile_list = s;
+      else
+	ofile->symbol_list_head = s;
+      ofile->symbol_list_tail = s;
+      /* And the block list */
+    }
   if (b->vars_tail)
     b->vars_tail->next = s;
   else
@@ -539,14 +565,15 @@ do_define (i, b)
   if (s->where->where == coff_where_memory
       && s->type->type == coff_secdef_type)
     {
-      struct coff_isection *is= cur_sfile->section + s->where->section->number;
-      
-      if (!is->init) {
-	is->low = s->where->offset;
-	is->high = s->where->offset + s->type->size;
-	is->init = 1;
-	is->parent = s->where->section;
-      }
+      struct coff_isection *is = cur_sfile->section + s->where->section->number;
+
+      if (!is->init)
+	{
+	  is->low = s->where->offset;
+	  is->high = s->where->offset + s->type->size;
+	  is->init = 1;
+	  is->parent = s->where->section;
+	}
 
     }
 
@@ -563,11 +590,14 @@ doit ()
 {
   int i;
   int infile = 0;
-  struct coff_ofile *head = (struct coff_ofile *) malloc (sizeof (struct coff_ofile));
+  struct coff_ofile *head =
+  (struct coff_ofile *) xmalloc (sizeof (struct coff_ofile));
   ofile = head;
   head->source_head = 0;
   head->source_tail = 0;
-
+  head->nsources = 0;
+  head->symbol_list_tail = 0;
+  head->symbol_list_head = 0;
   do_sections_p1 (head);
   push_scope (1);
 
@@ -579,8 +609,9 @@ doit ()
 	case C_FILE:
 	  {
 	    /* new source file announced */
-	    struct coff_sfile *n = (struct coff_sfile *) malloc (sizeof (struct coff_sfile));
-	    n->section = (struct coff_isection *)xcalloc (sizeof (struct coff_isection), abfd->section_count+1);
+	    struct coff_sfile *n =
+	      (struct coff_sfile *) xmalloc (sizeof (struct coff_sfile));
+	    n->section = (struct coff_isection *) xcalloc (sizeof (struct coff_isection), abfd->section_count + 1);
 	    cur_sfile = n;
 	    n->name = sym->_n._n_nptr[1];
 	    n->next = 0;
@@ -701,6 +732,6 @@ coff_grok (inabfd)
   rawcount = obj_raw_syment_count (abfd);;
   tindex = (struct coff_symbol **) (xcalloc (sizeof (struct coff_symbol *), rawcount));
 
-  p = doit (abfd);
+  p = doit ();
   return p;
 }
